@@ -12,21 +12,19 @@
 #'
 #' The latent class IRT model with two latent classes has the form
 #'
-#' \deqn{P(x = k|\theta_1, \theta_2, a1, a2) = \frac{exp(s_k (a1 \theta_1 + a2 \theta_2))}{
-#'   \sum_j^K exp(s_j (a1 \theta_1 + a2 \theta_2))}}
+#' \deqn{P(x = k|\theta_1, \theta_2, a1, a2) = \frac{exp(a1 \theta_1 + a2 \theta_2)}{
+#'   \sum_j^K exp(a1 \theta_1 + a2 \theta_2)}}
 #'
-#' where the \eqn{\theta} values generally take on discrete points (such as 0 or 1), and
-#' the \eqn{s_k}'s are the scoring values for each category. If the model is selected to be
-#' \code{'lca'} then the \eqn{s_k} values are fixed to \code{s_k = 0:(ncat - 1)}, whereas if
-#' the model is \code{'nlca'} the \eqn{s_k} are all fixed to 1. For proper identification, the
-#' first category slope parameters (\eqn{a1} and \eqn{a2}) are never freely estimated.
+#' where the \eqn{\theta} values generally take on discrete points (such as 0 or 1).
+#' For proper identification, the first category slope parameters
+#' (\eqn{a1} and \eqn{a2}) are never freely estimated. Alternatively, supplying a different
+#' grid of \eqn{\theta} values will allow the estimation of similar models (multidimensional
+#' discrete models, grade of membership, etc.). See the examples below.
 #'
 #' @param data a \code{matrix} or \code{data.frame} that consists of
 #'   numerically ordered data, with missing data coded as \code{NA}
 #' @param model number of classes to fit, or alternatively a \code{\link{mirt.model}} definition
-#' @param itemtype item types to use. Can be the \code{'lca'} model for defining ordinal
-#'   item response models (dichotomous items are a special case), \code{'nlca'} for the
-#'   unordered latent class model
+#' @param method estimation method. Can be 'EM' or 'BL' (see \code{\link{mirt}} for more details)
 #' @param group a factor variable indicating group membership used for multiple group analyses
 #' @param GenRandomPars logical; use random starting values
 #' @param technical technical input list, most interesting for discrete latent models
@@ -116,17 +114,18 @@
 #' summary(mod_discrete)
 #'
 #' }
-mdirt <- function(data, model, itemtype = 'lca', nruns = 1,
+mdirt <- function(data, model, nruns = 1, method = 'EM',
                   return_max = TRUE, group = NULL, GenRandomPars = FALSE,
                   verbose = TRUE, pars = NULL, technical = list(), ...)
 {
     Call <- match.call()
-    if(!all(itemtype %in% c('lca', 'nlca')))
-        stop('Selected itemtype not supported. Please use itemtype \'lca\' or \'nlca\'', call.=FALSE)
+    itemtype <- 'lca'
+    stopifnot(method %in% c('EM', 'BL'))
     if(nruns > 1) GenRandomPars <- TRUE
     if(is.null(group)) group <- rep('all', nrow(data))
-    mods <- myLapply(1:nruns, function(x, ...) return(ESTIMATION(...)),
-                     data=data, model=model, group=group, itemtype=itemtype, method='EM',
+    stopifnot(is.numeric(model))
+    mods <- myLapply(1:nruns, function(x, ...) return(ESTIMATION(...)), method=method,
+                     data=data, model=model, group=group, itemtype=itemtype,
                      technical=technical, calcNull=FALSE, GenRandomPars=GenRandomPars,
                      discrete=TRUE, verbose=ifelse(nruns > 1L, FALSE, verbose), pars=pars, ...)
     if(is(mods[[1L]], 'DiscreteClass')){
